// Info Viewer V1.0 Copyright (C) 1997 Wen-Shin Wang

/* This file is part of RHIDE, Copyright (C) 1996 Robert Hhne */
/***************************************************************

   (c) 1996 by Salvador Eduardo Tropea (SET).
   e-mail: salvador@inti.edu.ar, ice@inti.edu.ar

   You can use this code for any purpose if you don't remove the
original copyright.

 Filename -		  InfBase.h

 Functions
				  Member functions of following classes
					  TInfTopic
					  TInfIndex
					  TInfFile

***************************************************************/

#ifndef __InfBase_H
#define __InfBase_H

#define cInfViewer	   "\x06\x07\x08\x09"
#define cInfWindowColor "\x6A\x6B\x6C\x6D\x6E\x6F\x70\x71\x72\x73"
#define cInfWindowBW	"\x74\x75\x76\x77\x78\x79\x7A\x7B\x7C\x7D"
#define cInfWindowMono	"\x7E\x7F\x80\x81\x82\x83\x84\x85\x86\x87"

#include <stdio.h>

#define MAX_NODE_NAME 90
#define BUF_SIZE (MAX_NODE_NAME*4+MAX_NODE_NAME/2)

// structure CrossRef

struct CrossRef
{
 char Name[MAX_NODE_NAME];
 char Name2[MAX_NODE_NAME];
 int offset;
 int line;
 int linebreak;
 uchar length;
};


// structure IndirectIndex

typedef struct
{
 long lPos;
 char Name[MAX_NODE_NAME];
} IndirectIndex;


// TInfTopic

class TInfFile;

class TInfTopic
{

public:

	TInfTopic();
	virtual ~TInfTopic();

	void getCrossRef( int i, TPoint& loc, uchar& length, char *& ref, int &pl );
	char *getCrossRef( int i ) { return crossRefs[i].Name; };
	int getLine( int line, char *buffer );
	int getNumCrossRefs() { return numRefs; };
	int numLines() { return iLines; };
	int maxWidth() { return maxLineWidth; };
	void setWidth( int aWidth ) { width = aWidth; };

	int selBestMatch(char *match,int &PerfectMatch);

	int numRefs;
	CrossRef *crossRefs;

	Boolean Status;

	void *Read( TInfFile& File	);

	// Lo que contiene
	char *Text;
	int	 iLines;
	int	 maxLineWidth;
	long lSize;

	char Node[MAX_NODE_NAME];
	char Next[MAX_NODE_NAME];
	char Prev[MAX_NODE_NAME];
	char Up[MAX_NODE_NAME];

	int SearchNextWith(char *str,int len,int &selected,int &vislen);

	void ReadNodeInfo(TInfFile &File);

private:

	void ReadText( TInfFile &File );
	void ReadCrossRefs( void );
	int width;

	// Usadas por getLine
	int lastOffset;
	int lastLine;

};

// Valores para modeFlag
extern int modeFlags;
const modeCutNodeWord  = 1,
	  modeHideNodeLink = 2;

// TInfIndex

typedef char NameOfNode[MAX_NODE_NAME];

class TInfIndex
{
public:

	TInfIndex( TInfFile& o, int Indirect );
	~TInfIndex();

	long position( int );
	int WhatIndex(char *Nom);

	int size;
	NameOfNode *Names;
	long *Offsets;
	IndirectIndex *IndOffsets;
	int indirects;
	Boolean Status;

	int SearchFirstWith(int Key,int &selected);
};


// TInfFile

class TInfFile
{

public:

	TInfFile( char *s ) { DoAll(s,0); };
//	  TInfFile( char *s ) { DoAll(s,1); };
	virtual ~TInfFile();
	void DoAll(char *s, int Verbose=0 );

	void seekToNode( const char *Name);
	TInfTopic *getTopic( char *, int Verbose=0 );
//	  TInfTopic *getTopic( char *, int Verbose=1 );
	TInfTopic *invalidTopic();

	FILE *stream;
	Boolean Status;
	Boolean IsCompressed;

	TInfIndex *index;
	long indexPos;
	int	 iFile;

	char Buffer[BUF_SIZE];
	void GetLine(void) { fgets(Buffer,SizeOfReadBuf,stream); };
	void SkipNode(void);

	// Special I/O for multi-file
	int	 fSeek(long Pos);
	FILE *fOpen(char *Nombre);
	int fClose(FILE *f);

	char NameOfFile[MAX_NODE_NAME];

	int ConvertIt(long Pos);
	static int SizeOfReadBuf;
	char NameOfTemporal[L_tmpnam];

private:
	int ExpandName(char *Buf, char *Nombre, int iExt);

};

#endif // __InfBase_H


