// Info Viewer V1.0 Copyright (C) 1997 Wen-Shin Wang

#include "tdef.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <go32.h>
#include <sys/exceptn.h>
#include <dir.h>
#include <unistd.h>
#include "getopt.h"
#include "inf.h"
#include "iv.h"
#include "gte32.h"

#ifdef asciichart
#include "ascii.h"
#endif

extern int gteGraphicText;
#define GT gteGraphicText
extern int gteGraphicMouse;
#define GM gteGraphicMouse

int WindowCount=0;
char *ClipFile="IVCLIP.TMP";

TIVInfWindow::TIVInfWindow(TInfFile *hFile, char *context, char *match):
	TInfWindow(hFile,context,match),TWindowInit( &TIVInfWindow::initFrame) {
	WindowCount++;
}

TIVInfWindow::~TIVInfWindow() {
	if (--WindowCount<=0) exit(0);
}

TInfWindow* InfoView() {
	return new TIVInfWindow(new TInfFile("dir"),"");
}

static char EXEName[MAXFILE],*NodeToSearch=0;
static void help() {
	fprintf (stderr,
		"usage: %s [options] [(Inf File) [topic]]\n"
		"options:\n"
		"   -?      : This help.\n"
		"   -c      : Cut '*Note' words.\n"
		"   -g +/-  : Force/don't use graphic text mode.\n"
		"   -h      : Hide node links.\n"
		"   -m mode : Use extended video mode. (default:default)\n"
		"           : default,80x30,80x34,90x30,90x34,94x30,94x34\n"
		"   -n node : Search node in startup topic.\n"
		"             (just like typing them in one by one)\n"
		"   -o name : The file to store marked context. will be deleted at startup.\n"
		"             default: IVCLIP.TMP\n"
		"\n"
		"eg. %s -m94x34 (libc)Alphabetical@List -n abort\n"
		"eg. %s -c -h (binutils) ranlib -g- \n"
		,EXEName,EXEName,EXEName
	);
	_exit(0);
}


void ProceedArgs(int &argc,char** &argv) {
int c;
char *optchars="chm:g:n:o:?",o[2]=" ";
char drive[MAXDRIVE],dir[MAXDIR],name[MAXFILE],ext[MAXEXT];
char *gteModeName[]={"default","80x30","80x34","90x30","90x34","94x30","94x34"};

	fnsplit(argv[0],drive,dir,name,ext);
	strcpy(EXEName,name);
	strcat(EXEName,ext);
	gteGraphicText=1;
	gteGraphicMouse=1;
	gteScreenMode=gteDefault;

	while (1) {
		c=getopt(argc,argv,optchars);
		if (c==-1) break;
		switch(c) {
			case 'c': modeFlags|= modeCutNodeWord;
			break;

			case 'h': modeFlags|= modeHideNodeLink;
			break;

			case 'g':
				if (optarg[0]=='-') {
					gteGraphicText=0;
					gteGraphicMouse=0;
				}
				else if (optarg[0]=='+') {
					gteForceGraphic=1;
				}
			break;

			case 'm':
			int i;
				for (i=0;i<7;i++) {
					if (stricmp(optarg,gteModeName[i])==0) {
						gteScreenMode=i;
						break;
					}
				}
			break;

			case 'n':
				if (!NodeToSearch) NodeToSearch=optarg;
			break;

			case 'o':
				ClipFile=optarg;
			break;

			case '?':
				o[0]=optopt;
				if ((optopt=='?')||(strstr(optchars,o)==NULL)) help();
			break;

			default:;
		}
	}
	if (optind < argc) {
		argc-=--optind;
		argv[optind]=argv[0];
		argv=&argv[optind];
	}
}

TInfoViewer::TInfoViewer(int argc,char* argv[]) : TProgInit(&TInfoViewer::initStatusLine,&TInfoViewer::initMenuBar,&TInfoViewer::initDeskTop)
{
TRect r;
TInfWindow *w;
TInfTopic *aTopic;
int i,sel,qpos;
size_t n,orin;
char *arg;

	unlink(ClipFile);
	r = getExtent();
	r.a.x=r.b.x-9;r.b.y=r.a.y+1;
	clock = new TClockView(r);
	insert(clock);
	r = getExtent();
	r.a.x=r.b.x-14;r.a.y=r.b.y-1;
	heap = new THeapView(r);
	insert(heap);

	w= InfoView();
	r.a=(TPoint){0,0};r.b=deskTop->size;
	w->changeBounds(r);

	argc--;argv=&argv[1];
	if (argc>0) for (i=0;i<argc;i++) {
		for (arg=argv[i],n=0;arg[n]!=0;n++) if (arg[n]=='@') arg[n]=' ';
		aTopic=w->viewer->hFile->getTopic(argv[i]);
		if (strstr(aTopic->Text,"\n Sorry")==aTopic->Text) break;
		else w->viewer->switchToTopic(argv[i]);
	}
	if (NodeToSearch) {
		orin=strlen(NodeToSearch);
		while (!isalpha(*NodeToSearch)) NodeToSearch++;
		n=strlen(NodeToSearch);
		if ((w->viewer->topic->numRefs)&&(n>0)) {
			aTopic=w->viewer->topic;
			for (i=1;(size_t)i<=n;i++) if (!aTopic->SearchNextWith(NodeToSearch,i,sel,qpos)) break;
			if ((sel) && ( (orin==strlen(aTopic->crossRefs[sel-1].Name)) || (orin==strlen(aTopic->crossRefs[sel-1].Name2)) )) {
				w->viewer->QuickLen=0;
				w->viewer->switchToTopic(aTopic->getCrossRef(sel-1));
			}
			else {
				strcpy(w->viewer->QuickSearch,NodeToSearch);
				w->viewer->QuickLen=i-1;
				w->viewer->selected=sel;
				w->viewer->QuickVisPos=qpos;
				w->viewer->makeSelectVisible();
			}
		}
	}
	deskTop->insert(w);
}

void doDestroy( TView* p, void *lR ) {
	if (lR); // avoid unused warning
	delete p;
}

TInfoViewer::~TInfoViewer() {
	deskTop->forEach(doDestroy,0);
}

void TInfoViewer::idle()
{
	TProgram::idle();
	clock->update();
	heap->update();
}


void TInfoViewer::About()
{
	TDialog *a = new TDialog(TRect(0,0,40,13), "About");
	a->insert(
		new TStaticText(TRect(2,1,38,12),
			"\n\n\n\n"
			"\3??? What do you want to know ???\n"
			"\3 There is no answer whatsoever!\n"
		)
	);
	a->insert(
		new TButton(TRect(14, 10, 26, 12), "OK", cmOK, bfDefault)
	);
	a->options |= ofCentered;
	deskTop->execView(a);
	destroy(a);
}

int ExportText(TInfViewer* viewer) {
char *buf;
FILE *f;
TInfTopic *topic;
int i,count=0,r,endLine;

	if (!viewer->TextSelected) return 0;
	topic=viewer->topic;
	f=fopen(ClipFile,"at");
	if (!f) return 0;
	buf = (char*)malloc(topic->maxWidth()+3);
	if (!buf) {
		fclose(f);
		return 0;
	}
	endLine=viewer->LineEndSelect;
	for (i=viewer->LineStartSelect;i<=endLine;i++) {
		topic->getLine(i,buf);
		strcat(buf,"\n");
		r=fwrite(buf,1,strlen(buf),f);
		if (r==-1) break;
		count+=r;
	}
	strcpy(buf,"--\n");
	fwrite(buf,1,strlen(buf),f);
	free(buf);
	fclose(f);
	return count;
}


void TInfoViewer::handleEvent(TEvent& event)
{
	TApplication::handleEvent(event);
	switch (event.what) {
		case evCommand:
			switch( event.message.command ) {
				case cmFileOpen:
					deskTop->insert(InfoView());
				break;
				case cmAbout:
					About();
				break;
#ifdef asciichart
				case cmAsciiChart:
					deskTop->insert((TAsciiChart *) validView(new TAsciiChart));
				break;
#endif
				case cmInfCopy:
					ExportText((TInfViewer*)event.message.infoPtr);
				break;
			}
			default:return;
			clearEvent(event);
		break;
	}
}


TMenuBar* TInfoViewer::initMenuBar( TRect r )
{
	r.b.y=r.a.y+1;
	return new TMenuBar( r,
		*new TSubMenu( "",kbNoKey )+
			*new TMenuItem( "~A~bout", cmAbout,   kbNoKey,   hcNoContext)+
#ifdef asciichart
			*new TMenuItem( "Ascii ~C~hart", cmAsciiChart,   kbNoKey,   hcNoContext)+
#endif
		*new TSubMenu( "~F~ile",kbAltF )+
			*new TMenuItem( "New ~V~iewer", cmFileOpen,   kbF3,   hcNoContext, "F3" )+
			newLine()+
			*new TMenuItem( "E~x~it", cmQuit, kbAltX, hcNoContext, "Alt+X" )+

		*new TSubMenu( "~E~dit",kbAltE )+
			*new TMenuItem( "~C~opy",  cmCopy,     kbNoKey,    hcNoContext,    "Ctrl+C")+
			newLine()+
			*new TMenuItem( "~F~ind",  cmFind,     kbNoKey,    hcNoContext,    "Ctrl+S")+
			*new TMenuItem( "Find ~a~gain",  cmSearchAgain,     kbNoKey,    hcNoContext,    "Ctrl+L")+

		*new TSubMenu( "~C~ontrol",kbAltC )+
			*new TMenuItem( "~B~ack",    cmInfBack,    kbNoKey,    hcNoContext,    "Ctrl+B")+
			*new TMenuItem( "~U~p",    cmInfUp,     kbNoKey,    hcNoContext,    "Ctrl+U")+
			*new TMenuItem( "~T~op",    cmInfTop,     kbNoKey,    hcNoContext,    "Ctrl+T")+
			*new TMenuItem( "~P~revious",    cmInfPrev,     kbNoKey,    hcNoContext,    "Ctrl+P")+
			*new TMenuItem( "~N~ext",    cmInfNext,     kbNoKey,    hcNoContext,    "Ctrl+N")+
			*new TMenuItem( "~I~ndex",   cmInfIndex,    kbNoKey,    hcNoContext,    "Ctrl+I")+
			*new TMenuItem( "~D~ir", cmInfDir,  kbNoKey,   hcNoContext,    "Ctrl+D")+

		*new TSubMenu( "~W~indow",  kbAltW )+
			*new TMenuItem( "~S~ize/move",  cmResize,   kbCtrlF5,   hcNoContext,    "Ctrl+F5" ) +
			*new TMenuItem( "~Z~oom",       cmZoom,     kbF5,       hcNoContext,    "F5" ) +
//			  *new TMenuItem( "~T~ile",       cmTile,     kbNoKey,    hcNoContext ) +
//			  *new TMenuItem( "~C~ascade",    cmCascade,  kbNoKey,    hcNoContext ) +
			*new TMenuItem( "~N~ext",       cmNext,     kbF6,       hcNoContext,    "F6" ) +
			*new TMenuItem( "~P~revious",   cmPrev,     kbShiftF6,  hcNoContext,    "Shift+F6" ) +
			*new TMenuItem( "~C~lose",      cmClose,    kbAltF3,    hcNoContext,    "Alt+F3" )+
			newLine()+
			*new TMenuItem( "~H~istory...", cmInfHistory,  kbNoKey,   hcNoContext,    "Ctrl+H")+
			*new TMenuItem( "~G~oto...", cmInfGoto,  kbNoKey,   hcNoContext,    "Ctrl+G")
	);
}

TStatusLine* TInfoViewer::initStatusLine( TRect r )
{
	r.a.y=r.b.y-1;
	return new TStatusLine( r,
		*new TStatusDef( 0, 0xFFFF ) +
			*new TStatusItem( 0, kbF10, cmMenu ) +
			*new TStatusItem( "~Alt-X~ Exit", kbAltX, cmQuit )
	);
}


TPalette& TInfoViewer::getPalette() const
{
	static TPalette palette(cpTInfoViewerColor,sizeof(cpTInfoViewerColor)-1);
	return palette;
}

static void iv(int argc,char** argv) {
TInfoViewer TInfoViewer(argc,argv);
	TInfoViewer.run();
}

int main(int argc,char* argv[])
{
	TEventQueue::suspend();
	TScreen::suspend();
	fprintf(stderr,"Info Viewer 1.0 c-in-circle 1997 by Wen-Shin Wang\n");
	ProceedArgs(argc,argv);
	TScreen::resume();
	TEventQueue::resume();
	_go32_want_ctrl_break(1);
	__djgpp_set_ctrl_c(0);
	iv(argc,argv);
	return 0;
}
